@@ -8,6 +8,8 @@
 namespace pyalign {
 namespace core {
 
+typedef uint16_t layer_t;
+
 namespace goal {
 	struct optimal_score { // compute only optimal score
 		struct path_goal {
@@ -389,14 +391,16 @@ struct custom_batch_size {
 struct machine_batch_size {
 	template<typename Value, typename Index>
 	struct size {
-#if __AVX512F__
-		static constexpr int s = 512 / (std::max(sizeof(Value), sizeof(Index)) * 8);
+#if __aarch64__
+    static constexpr int s = 128 / (std::max(sizeof(Value), sizeof(Index)) * 8);
+#elif __AVX512F__
+    static constexpr int s = 512 / (std::max(sizeof(Value), sizeof(Index)) * 8);
 #elif __AVX2__
-		static constexpr int s = 256 / (std::max(sizeof(Value), sizeof(Index)) * 8);
+    static constexpr int s = 256 / (std::max(sizeof(Value), sizeof(Index)) * 8);
 #elif __SSE__
-		static constexpr int s = 128 / (std::max(sizeof(Value), sizeof(Index)) * 8);
+    static constexpr int s = 128 / (std::max(sizeof(Value), sizeof(Index)) * 8);
 #else
-		static constexpr int s = 1;
+    static constexpr int s = 1;
 #endif
 	};
 };
@@ -474,7 +478,7 @@ protected:
 	const std::unique_ptr<Data> m_data;
 	const size_t m_max_len_s;
 	const size_t m_max_len_t;
-	const uint16_t m_layer_count;
+	const layer_t m_layer_count;
 
 	inline void check_size_against_max(
 		const char *p_name,
@@ -498,7 +502,7 @@ public:
 	inline MatrixFactory(
 		const size_t p_max_len_s,
 		const size_t p_max_len_t,
-		const uint16_t p_layer_count) :
+		const layer_t p_layer_count) :
 
 		m_data(std::make_unique<Data>()),
 		m_max_len_s(p_max_len_s),
@@ -621,14 +625,14 @@ private:
 	const MatrixFactory<CellType, ProblemType> &m_factory;
 	const index_type m_len_s;
 	const index_type m_len_t;
-	const uint16_t m_layer;
+	const layer_t m_layer;
 
 public:
 	inline Matrix(
 		const MatrixFactory<CellType, ProblemType> &factory,
 		const index_type len_s,
 		const index_type len_t,
-		const uint16_t layer) :
+		const layer_t layer) :
 
 	    m_factory(factory),
 	    m_len_s(len_s),
