@@ -1,10 +1,8 @@
 import numpy as np
 import time
 import contextlib
-import importlib
 import typing
-import os
-import logging
+import itertools
 import pyalign.io.alignment
 
 from cached_property import cached_property
@@ -14,35 +12,7 @@ from collections.abc import Sequence
 
 from .gaps import GapCost, ConstantGapCost
 from .problems import ProblemBag, Form
-
-
-def has_avx2():
-	import cpufeature
-	return cpufeature.CPUFeature["AVX2"]
-
-
-def import_algorithm():
-	if os.environ.get('PYALIGN_PDOC') is not None:
-		return None
-
-	candidates = (
-		('native', lambda: True),
-		('avx2', has_avx2),
-		('generic', lambda: True)
-	)
-
-	missing_module_names = []
-
-	for name, check in candidates:
-		module_name = f"pyalign.algorithm.{name}"
-		if importlib.util.find_spec(module_name) is not None:
-			if check():
-				logging.info(f"running in {name} mode.")
-				return importlib.import_module(module_name + ".algorithm")
-		else:
-			missing_module_names.append(module_name)
-
-	raise RuntimeError(f"failed to find suitable c++ core in {missing_module_names}")
+from .cpu import import_algorithm
 
 
 algorithm = import_algorithm()
@@ -172,16 +142,28 @@ def next_power_of_2(x):
 	return 1 if x == 0 else 2 ** (int(x) - 1).bit_length()
 
 
+class SizeRounder:
+	def __init__(self, limit=1024):
+		self._limit = limit
+
+	def __call__(self, x):
+		if x >= self._limit:
+			return x
+		else:
+			return next_power_of_2(x)
+
+
 class SolverCache:
 	def __init__(self, options):
 		self._options = options
 		self._max_lim_s = 0
 		self._max_lim_t = 0
 		self._solvers = {}
+		self._rounder = SizeRounder()
 
 	def ensure(self, len_s, len_t):
-		lim_s = max(self._max_lim_s, next_power_of_2(len_s))
-		lim_t = max(self._max_lim_t, next_power_of_2(len_t))
+		lim_s = max(self._max_lim_s, self._rounder(len_s))
+		lim_t = max(self._max_lim_t, self._rounder(len_t))
 
 		if lim_s > self._max_lim_s or lim_t > self._max_lim_t:
 			self._max_lim_s = lim_s
@@ -199,9 +181,14 @@ class SolverCache:
 			options['batch'] = batch
 			options['return_dup'] = options.get('return_dup', False)
 
-			parsed_options = algorithm.create_options(options)
+			if self._max_lim_s < 16000 and self._max_lim_t < 16000:
+				bits_algo = algorithm.m16
+			else:
+				bits_algo = algorithm.m32
 
-			solver = algorithm.create_solver(
+			parsed_options = bits_algo.create_options(options)
+
+			solver = bits_algo.create_solver(
 				self._max_lim_s, self._max_lim_t, parsed_options)
 
 			self._solvers[batch] = solver
@@ -297,6 +284,9 @@ class Iterator:
 		self._solver = solver
 		self._iterator = iterator
 
+	def is_exhausted(self):
+		return self._iterator.next() is None
+
 	def __iter__(self):
 		while True:
 			x = self._iterator.next()
@@ -313,9 +303,17 @@ class SolutionIterator(Iterator):
 	_element_class = Solution
 
 
+max_results = 100
+
+
 def make_list_factory(iterator):
 	def to_list(*args, **kwargs):
-		return list(iterator(*args, **kwargs))
+		it = iterator(*args, **kwargs)
+		r = list(itertools.islice(it, max_results))
+		if not it.is_exhausted():
+			raise RuntimeError(
+				f"more than {max_results} results, please use an iterator")
+		return r
 
 	return to_list
 
@@ -530,7 +528,7 @@ class Solver:
 		that can get processed in a single SIMD call on this machine.
 		"""
 		return self._cache.get(
-			1, 1, direction=algorithm.Direction.MAXIMIZE, batch=True).batch_size
+			1, 1, direction="maximize", batch=True).batch_size
 
 	def timings(self):
 		return Timings(self)
